package com.iot.farm;
 
import android.app.Activity;
import android.app.Dialog;
import android.app.ProgressDialog;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter; 
import android.os.Bundle;  
import android.os.Handler;
import android.widget.Button;
import android.widget.CompoundButton;
import android.widget.RadioButton;
import android.widget.RadioGroup; 
import android.widget.Switch;
import android.widget.TimePicker;
import android.widget.Toast;
import android.widget.CompoundButton.OnCheckedChangeListener;
import android.view.View;
import android.view.Window;
import android.view.View.OnClickListener; 

public class SettingActivity extends Activity {

	private TimePicker tpicker_time;
	private RadioGroup group_period;
	private RadioButton button_every_day;
	private RadioButton button_2_days;
	private RadioButton button_3_days;
	private Switch Switch_water_supply;
	private Switch Switch_water_fan;
	private Switch Switch_water_motor;
	private Switch Switch_water_led;
	private Button button_history;
	private Button button_confirm;
	private Button button_cancel;
	
	private boolean _reading = false;
	private ProgressDialog _loading;

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		requestWindowFeature(Window.FEATURE_NO_TITLE);
		setContentView(R.layout.activity_setting);

		tpicker_time = (TimePicker) findViewById(R.id.TPICKER_WATERSUPPLY);
		group_period = (RadioGroup) findViewById(R.id.RGROUP_PERIOD);
		Switch_water_supply = (Switch) findViewById(R.id.ID_WATER_SUPPLY);
		Switch_water_fan = (Switch) findViewById(R.id.ID_WATER_FAN);
		Switch_water_motor = (Switch) findViewById(R.id.ID_WATER_MOTOR);
		Switch_water_led = (Switch) findViewById(R.id.ID_WATER_LED);

		button_history = (Button) findViewById(R.id.ID_SETTING_HISTORY);
		button_confirm = (Button) findViewById(R.id.ID_SETTING_CONFIRM);
		button_cancel = (Button) findViewById(R.id.ID_SETTING_CANCEL);

		button_every_day = (RadioButton) findViewById(R.id.RB_EVERYDAY);
		button_2_days = (RadioButton) findViewById(R.id.RB_PER_2_DAYS);
		button_3_days = (RadioButton) findViewById(R.id.RB_PER_3_DAYS);
				
		_loading = new ProgressDialog(SettingActivity.this);
		_loading.setProgressStyle(ProgressDialog.STYLE_SPINNER);
		_loading.setMessage(" ҷ Դϴ.");
	//	timerDelayRemoveDialog(Constants.DELAY_TIME, _loading);
		_loading.setCancelable(false);
		
		Switch_water_supply.setOnCheckedChangeListener(new OnCheckedChangeListener() {
			
			@Override
			public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
				// TODO Auto-generated method stub
				set_device_status((IoTUtility.mkcommand_set_water_supply(isChecked)));
			} 
		});

		Switch_water_fan.setOnCheckedChangeListener(new OnCheckedChangeListener() {

			@Override
			public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
				// TODO Auto-generated method stub
				set_device_status((IoTUtility.mkcommand_set_water_fan(isChecked)));
			} 
		});
		
		Switch_water_motor.setOnCheckedChangeListener(new OnCheckedChangeListener() {
			
			@Override
			public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
				// TODO Auto-generated method stub
				set_device_status((IoTUtility.mkcommand_set_water_motor(isChecked)));
			} 
		});

		Switch_water_led.setOnCheckedChangeListener(new OnCheckedChangeListener() {

			@Override
			public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
				// TODO Auto-generated method stub
				set_device_status((IoTUtility.mkcommand_set_water_led(isChecked)));
			} 
		});
		
		button_history.setOnClickListener(new OnClickListener() {

			@Override
			public void onClick(View v) {
				// TODO Auto-generated method stub
				get_sensor_history();
			}
		});

		
		button_confirm.setOnClickListener(new OnClickListener() {

			@Override
			public void onClick(View v) {
				// TODO Auto-generated method stub
				set_info_to_server();
			}
		});

		button_cancel.setOnClickListener(new OnClickListener() {

			@Override
			public void onClick(View v) {
				// TODO Auto-generated method stub
				finish();
			}
		});

		get_info_from_server();

	}
	@Override
	public synchronized void onPause() {
		super.onPause(); 
		unregisterReceiver(mBroadCastReceiver);
	}

	@Override
	public synchronized void onResume() {
		super.onResume(); 
		registerReceiver(mBroadCastReceiver, makeIntentFilter());
	}

	private void set_device_status(byte[] _data)
	{
		if(!_reading){
			timerDelayRemoveDialog(Constants.DELAY_TIME, _loading);
			_loading.show();
			try {
				Thread.sleep(1000);
			} catch (InterruptedException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			TCPClient.getInstance().sendMessage(_data);
		}
	}
	
	private void get_sensor_history()
	{  
		Intent intent = new Intent(this, HistoryActivity.class); 
		startActivityForResult(intent, 0);
	}
	
	private void set_info_to_server()
	{   
		timerDelayRemoveDialog(Constants.DELAY_TIME, _loading);
		_loading.show();
		try {
			Thread.sleep(1000);
		} catch (InterruptedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		int radioButtonID = group_period.getCheckedRadioButtonId();
		View radioButton = group_period.findViewById(radioButtonID);
		int idx = group_period.indexOfChild(radioButton);
		TCPClient.getInstance().sendMessage(IoTUtility.mkcommand_set_setting(tpicker_time.getCurrentHour(), tpicker_time.getCurrentMinute(), idx)); 
	}

	private void get_info_from_server()
	{ 
		timerDelayRemoveDialog(Constants.DELAY_TIME, _loading);
		_reading = true;
		_loading.show();
		try {
			Thread.sleep(1000);
		} catch (InterruptedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		TCPClient.getInstance().sendMessage(IoTUtility.mkcommand_get_setting()); 
	}

	private static IntentFilter makeIntentFilter() {
		final IntentFilter intentFilter = new IntentFilter();  
		intentFilter.addAction(Constants.INTENTFILTER_DATA); 
		return intentFilter;
	}
	
	public void timerDelayRemoveDialog(long time, final Dialog d){
	    new Handler().postDelayed(new Runnable() {
	        public void run() {
	            //Toast.makeText(getApplicationContext(), " ҷ  ϴ.", Toast.LENGTH_SHORT).show();       
	            d.dismiss();   
	        }
	    }, time); 
	}


	private final BroadcastReceiver mBroadCastReceiver = new BroadcastReceiver() {
		@Override
		public void onReceive(Context context, Intent intent) {

			final String action = intent.getAction();

			if(Constants.INTENTFILTER_DATA.equals(action))
			{
				Bundle bundle = intent.getExtras();    

				if(bundle != null) 
				{  
					if(IoTUtility.is_get_setting(bundle.getString(Constants.INTENTEXTRA_RECEIVED_DATA)))
					{
						String[] _data = IoTUtility.get_setting(bundle.getString(Constants.INTENTEXTRA_RECEIVED_DATA));
						if(_data != null)
						{
							if(_data.length < 3)
							{
								
							}
							else 
							{ 

								tpicker_time.setCurrentHour(Integer.parseInt(_data[0]));
								tpicker_time.setCurrentMinute(Integer.parseInt(_data[1])); 
								
								switch(Integer.parseInt(_data[2]))
								{
									case 0 : button_every_day.setChecked(true); break;
									case 1 : button_2_days.setChecked(true); break;
									case 2 : button_3_days.setChecked(true); break;
								}
								 
							Switch_water_supply.setChecked(Integer.parseInt(_data[3]) == 1);
							Switch_water_fan.setChecked(Integer.parseInt(_data[4]) == 1);
							Switch_water_motor.setChecked(Integer.parseInt(_data[5]) == 1);
							Switch_water_led.setChecked(Integer.parseInt(_data[6]) == 1);
					 
							_loading.dismiss();
							_reading = false;
							}
						}
					}
 
					else if(IoTUtility.is_set_setting(bundle.getString(Constants.INTENTEXTRA_RECEIVED_DATA)))
					{
						Toast.makeText(getApplicationContext(), " ݿ Ǿϴ.",Toast.LENGTH_SHORT).show(); 
						_loading.dismiss();
					}
				}

			}
		}
	};
}
