package com.iot.farm;
 
import android.app.Activity;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter; 
import android.os.Bundle; 
import android.os.Handler;
import android.view.Window;
import android.widget.ImageView;
import android.widget.TextView;

public class StatusActivity extends Activity {

	private TextView mtview_temp;
	private TextView mtview_humid; 
	private TextView mtview_light_functionality;
	private TextView mtview_humid_functionality;
	private TextView mtview_temp_functionality;
	private TextView mtview_pump_functionality;
	private TextView mtview_fan_functionality;
	private TextView mtview_motor_functionality;
	private TextView mtview_led_functionality;

	private ImageView miview_light;
	
	private Handler mHandler;
	
	private boolean _is_wating;

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		requestWindowFeature(Window.FEATURE_NO_TITLE);
		setContentView(R.layout.activity_status);

		mtview_temp = (TextView)findViewById(R.id.ID_STATUS_AIR_TEMP);
		mtview_humid = (TextView)findViewById(R.id.ID_STATUS_AIR_HUMID); 
		miview_light = (ImageView)findViewById(R.id.ID_STATUS_LIGHT);
		mtview_light_functionality = (TextView)findViewById(R.id.TVIEW_SENSOR_HUMID);
		mtview_humid_functionality = (TextView)findViewById(R.id.TVIEW_SENSOR_HUMID);
		mtview_temp_functionality = (TextView)findViewById(R.id.TVIEW_SENSOR_TEMP);
		mtview_pump_functionality = (TextView)findViewById(R.id.TVIEW_WATER_PUMP);
		mtview_fan_functionality = (TextView)findViewById(R.id.TVIEW_WATER_FAN);
		mtview_motor_functionality = (TextView)findViewById(R.id.TVIEW_DCMOTOR);
		mtview_led_functionality = (TextView)findViewById(R.id.TVIEW_LED);

		//initial
		get_info_from_server();
		
		mHandler = new Handler();

		GetInfoThread _info_thread = new GetInfoThread(true);

		_info_thread.start();
		
	}

	@Override
	public synchronized void onPause() {
		super.onPause(); 
		unregisterReceiver(mBroadCastReceiver);
	}

	@Override
	public synchronized void onResume() {
		super.onResume(); 
		registerReceiver(mBroadCastReceiver, makeIntentFilter());
	}

	private void get_info_from_server()
	{ 
		if(!_is_wating)
		{
			_is_wating = true;
			TCPClient.getInstance().sendMessage(IoTUtility.mkcommand_get_status());
		}
	}

	private static IntentFilter makeIntentFilter() {
		final IntentFilter intentFilter = new IntentFilter();  
		intentFilter.addAction(Constants.INTENTFILTER_DATA); 
		return intentFilter;
	}


	private final BroadcastReceiver mBroadCastReceiver = new BroadcastReceiver() {
		@Override
		public void onReceive(Context context, Intent intent) {

			final String action = intent.getAction();

			if(Constants.INTENTFILTER_DATA.equals(action))
			{
				Bundle bundle = intent.getExtras();    

				if(bundle != null) 
				{ 
					String[] _data = IoTUtility.get_status(bundle.getString(Constants.INTENTEXTRA_RECEIVED_DATA));
					if(_data != null)
					{

						mtview_temp.setText(Integer.parseInt(_data[0]) + "");
						mtview_humid.setText(Integer.parseInt(_data[1]) + "%");

						if(Integer.parseInt(_data[2]) == 1)
							miview_light.setImageResource(R.drawable.lunar);
						else
							miview_light.setImageResource(R.drawable.sun);

						if(Integer.parseInt(_data[3]) == 1)
							mtview_light_functionality.setText(R.string.STR_NORMAL);
						else
							mtview_light_functionality.setText(R.string.STR_UNNORMAL);
						if(Integer.parseInt(_data[4]) == 1)
							mtview_humid_functionality.setText(R.string.STR_NORMAL);
						else
							mtview_humid_functionality.setText(R.string.STR_UNNORMAL);
						if(Integer.parseInt(_data[5]) == 1)
							mtview_temp_functionality.setText(R.string.STR_NORMAL);
						else
							mtview_temp_functionality.setText(R.string.STR_UNNORMAL);
						if(Integer.parseInt(_data[6]) == 1)
							mtview_pump_functionality.setText(R.string.STR_NORMAL);
						else
							mtview_pump_functionality.setText(R.string.STR_UNNORMAL);
						if(Integer.parseInt(_data[7]) == 1)
							mtview_fan_functionality.setText(R.string.STR_NORMAL);
						else
							mtview_fan_functionality.setText(R.string.STR_UNNORMAL);
						if(Integer.parseInt(_data[8]) == 1)
							mtview_motor_functionality.setText(R.string.STR_NORMAL);
						else
							mtview_motor_functionality.setText(R.string.STR_UNNORMAL);
						if(Integer.parseInt(_data[9]) == 1)
							mtview_led_functionality.setText(R.string.STR_NORMAL);
						else
							mtview_led_functionality.setText(R.string.STR_UNNORMAL);
						
						_is_wating = false;
					}
				}

			}

		}
	};
	
	class GetInfoThread extends Thread {
		private boolean isPlay = false;

		public GetInfoThread(boolean isPlay){
			this.isPlay = isPlay;
		}

		public void stopThread(){
			isPlay = !isPlay;
		}
		@Override
		public void run() {
			super.run();
			while (isPlay) {
				try { 
					Thread.sleep(1000);
				} catch (InterruptedException e) {
					e.printStackTrace();
				}
				mHandler.post(new Runnable() {
					@Override
					public void run() {
						get_info_from_server();
					}
				});
			}
		}
	}
}
